CREATE OR REPLACE PACKAGE pkg_consultes
IS
    RSP logs.RSP%TYPE;

    -- Mostra la llista de jugadors d'un competicio indicada
    PROCEDURE lst_jugadorsCompeticio(
        p_idCompeticio IN competicions.idCompeticio%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Retorna el llistat del equips de la competicio indicada
    -- ordenats per punts
    PROCEDURE lst_equipsCompeticio(
        p_idCompeticio IN competicions.idCompeticio%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Mostra el nombre de contractes per representant i any
    PROCEDURE lst_contractesRepresentant(
        p_any IN INTEGER,
        p_idRepresentant IN representantsJugador.idRepresentant%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Llista els 10 equips que mes s'han gastat en un any concret
    PROCEDURE lst_top10DespesaEquips(
        p_any IN INTEGER,
        RSP OUT logs.RSP%TYPE
    );

    -- Llista els 5 jugadors millor valorats per posicio segons ambit, genere i modalitat competicio
    PROCEDURE lst_top5Jugadors(
        p_ambit competicions.ambit%TYPE,
        p_genere competicions.genere%TYPE,
        p_modalitat competicions.modalitat%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Llistat de jugadors que acaben contracte o no en tenen segons pais, genere i modalitat
    PROCEDURE lst_jugadorsLliures(
        p_idPais IN paisos.idPais%TYPE,
        p_genere competicions.genere%TYPE,
        p_modalitat competicions.modalitat%TYPE,
        RSP OUT logs.RSP%TYPE
    );

END pkg_consultes;
/
CREATE OR REPLACE PACKAGE BODY pkg_consultes
IS

    -- Mostra la llista de jugadors d'un competicio indicada
    PROCEDURE lst_jugadorsCompeticio(
        p_idCompeticio IN competicions.idCompeticio%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS

        CURSOR c_jugadors IS
            SELECT persones.nom, persones.cognoms, persones.dataNaixement,
                   paisos.nom nomPais,
                   DECODE(persones.estat,'A','Alta','Baixa') estatPersona,
                   federacions.nom nomFederacio, jugadors.numFederat,
                   jugadors.alcada, jugadors.pes,
                   DECODE(jugadors.posicio,1,'Base',2,'Escorta',3,'Aler',4,'Aler-Pivot','Pivot') posicio,
                   jugadors.urlWeb, jugadors.urlVideo,
                   DECODE(jugadors.estat,'A','Alta','Baixa') estatJugador,
                   contractes.equipComprador, contractes.dataInici, contractes.dataFi,
                   contractes.salari, contractes.moneda
              FROM jugadors, contractes, persones, paisos, federacions
             WHERE jugadors.idPersona = persones.idPersona
               AND persones.idPais = paisos.idPais
               AND jugadors.idFederacio = federacions.idFederacio
               AND jugadors.idJugador = contractes.idJugador(+)
               AND CURRENT_DATE BETWEEN contractes.dataInici(+) AND contractes.dataFi(+)
               AND jugadors.idJugador IN (SELECT DISTINCT convocats.idJugador
                                            FROM partits, convocats
                                           WHERE convocats.idPartit = partits.idPartit
                                             AND partits.idCompeticio = p_idCompeticio);

        e_competicioNF EXCEPTION;
        m_nom persones.nom%TYPE;
        m_cognoms persones.cognoms%TYPE;
        m_dataNaixement persones.dataNaixement%TYPE;
        m_nomPais paisos.nom%TYPE;
        m_estatPersona VARCHAR2(5);
        m_nomFederacio federacions.nom%TYPE;
        m_numFederat jugadors.numFederat%TYPE;
        m_alcada jugadors.alcada%TYPE;
        m_pes jugadors.pes%TYPE;
        m_posicio VARCHAR2(10);
        m_urlWeb jugadors.urlWeb%TYPE;
        m_urlVideo jugadors.urlVideo%TYPE;
        m_estatJugador VARCHAR2(5);
        m_equipComprador contractes.equipComprador%TYPE;
        m_dataInici contractes.dataInici%TYPE;
        m_dataFi contractes.dataFi%TYPE;
        m_salari contractes.salari%TYPE;
        m_moneda contractes.moneda%TYPE;
        trobat NUMBER;

    BEGIN

        -- Control per determinar si existeix el ID de competicio
        SELECT COUNT(*) INTO trobat FROM competicions WHERE idCompeticio = p_idCompeticio;
        IF (trobat = 0) THEN
            RAISE e_competicioNF;
        END IF;

        -- Buffer del output
        DBMS_OUTPUT.ENABLE(1000);
        DBMS_OUTPUT.PUT_LINE('=======================');
        DBMS_OUTPUT.PUT_LINE('Jugadors per competicio');
        DBMS_OUTPUT.PUT_LINE('=======================');

        OPEN c_jugadors;
        -- Mostrem les dades de cada jugador
        LOOP
            FETCH c_jugadors INTO m_nom, m_cognoms, m_dataNaixement, m_nomPais, m_estatPersona,
                                  m_nomFederacio, m_numFederat, m_alcada, m_pes, m_posicio,
                                  m_urlWeb, m_urlVideo, m_estatJugador, m_equipComprador,
                                  m_dataInici, m_dataFi, m_salari, m_moneda;
            EXIT WHEN c_jugadors%NOTFOUND;

            DBMS_OUTPUT.PUT_LINE(m_nom || ', ' ||  m_cognoms || ', ' || m_dataNaixement || ', ' ||
                                 m_nomPais || ', ' || m_estatPersona || ', ' || m_nomFederacio || ', ' ||
                                 m_numFederat || ', ' || m_alcada || ', ' || m_pes || ', ' || m_posicio || ', ' ||
                                 m_urlWeb || ', ' || m_urlVideo || ', ' || m_estatJugador || ', ' || m_equipComprador || ', ' ||
                                 m_dataInici || ', ' || m_dataFi || ', ' || m_salari || ', ' || m_moneda);
        END LOOP;

        CLOSE c_jugadors;

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('lst_jugadorsCompeticio', SYSDATE, p_idCompeticio, RSP);

    EXCEPTION
        WHEN e_competicioNF THEN
            RSP := 'ERROR: Competicio [' || p_idCompeticio || '] no trobada';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_jugadorsCompeticio', SYSDATE, p_idCompeticio, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_jugadorsCompeticio', SYSDATE, p_idCompeticio, RSP);
    END lst_jugadorsCompeticio;

    -- Retorna el llistat del equips de la competicio indicada
    -- ordenats per punts
    PROCEDURE lst_equipsCompeticio(
        p_idCompeticio IN competicions.idCompeticio%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_competicioNF EXCEPTION;

        --Cursor per desar les dades dels equips i dels punts totals
        CURSOR c_equips IS
            SELECT equips.nom,
                   SUM(CASE WHEN partits.equipLocal = equips.idEquip THEN partits.puntsLocal
                            ELSE partits.puntsVisitant
                        END) punts
              FROM equips, partits, competicions
             WHERE ((equips.idEquip = partits.equipVisitant) OR
                   (equips.idEquip = partits.equipLocal))
               AND partits.idCompeticio = competicions.idCompeticio
               AND competicions.idCompeticio = p_idCompeticio
             GROUP BY equips.nom
             ORDER BY punts DESC;

        m_nom equips.nom%TYPE;
        m_punts partits.puntsLocal%TYPE;
        trobat NUMBER;

    BEGIN

        -- Control per determinar si existeix el ID de competicio
        SELECT COUNT(*) INTO trobat FROM competicions WHERE idCompeticio = p_idCompeticio;
        IF (trobat = 0) THEN
            RAISE e_competicioNF;
        END IF;

        -- Buffer del output
        DBMS_OUTPUT.ENABLE(1000);
        DBMS_OUTPUT.PUT_LINE('=====================');
        DBMS_OUTPUT.PUT_LINE('Equips per competicio');
        DBMS_OUTPUT.PUT_LINE('=====================');

        OPEN c_equips;
        --Per a cada equip, mostrar els punts
        LOOP
            FETCH c_equips INTO m_nom, m_punts;
            EXIT WHEN c_equips%NOTFOUND;

            DBMS_OUTPUT.PUT_LINE(m_nom || ': ' || m_punts);
        END LOOP;

        CLOSE c_equips;

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('lst_equipsCompeticio', SYSDATE, p_idCompeticio, RSP);

    EXCEPTION
        WHEN e_competicioNF THEN
            RSP := 'ERROR: ID de competicio no trobat';
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_equipsCompeticio', SYSDATE, p_idCompeticio, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_equipsCompeticio', SYSDATE, p_idCompeticio, RSP);

    END lst_equipsCompeticio;

    -- Mostra el nombre de contractes per representant i any
    PROCEDURE lst_contractesRepresentant(
        p_any IN INTEGER,
        p_idRepresentant IN representantsJugador.idRepresentant%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_representantNF EXCEPTION;

        --Cursor per desar les dades dels equips i dels punts totals
        CURSOR c_representants IS
           SELECT persones.nom, persones.cognoms,
                  COUNT(*) numContractes,
                  SUM(contractes.valorEconomic) valorEconomic
             FROM representantsJugador, persones, jugadors, contractes
            WHERE representantsJugador.idPersona = persones.idPersona
              AND jugadors.idJugador = representantsJugador.idJugador
              AND contractes.idJugador = jugadors.idJugador
              AND representantsJugador.idPersona = p_idRepresentant
              AND EXTRACT(YEAR FROM TO_DATE(contractes.dataInici)) = p_any
            GROUP BY persones.nom, persones.cognoms;

        m_nom persones.nom%TYPE;
        m_cognoms persones.cognoms%TYPE;
        m_numContractes INTEGER;
        m_valorEconomic DECIMAL(16,2);
        trobat NUMBER;
    BEGIN
        -- Control per determinar si existeix el ID de representant
        SELECT COUNT(*) INTO trobat FROM representantsJugador WHERE idRepresentant = p_idRepresentant;
        IF (trobat = 0) THEN
            RAISE e_representantNF;
        END IF;

        -- Buffer del output
        DBMS_OUTPUT.ENABLE(1000);
        DBMS_OUTPUT.PUT_LINE('=================================');
        DBMS_OUTPUT.PUT_LINE('Contractes per representant i any');
        DBMS_OUTPUT.PUT_LINE('=================================');

        OPEN c_representants;
        --Per a cada representant, mostrar les dades dels contractes
        LOOP
            FETCH c_representants INTO m_nom, m_cognoms, m_numContractes, m_valorEconomic;
            EXIT WHEN c_representants%NOTFOUND;

            DBMS_OUTPUT.PUT_LINE(m_nom || ', ' || m_cognoms || ', ' || m_numContractes || ', ' || m_valorEconomic);
        END LOOP;

        CLOSE c_representants;

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('lst_contractesRepresentant', SYSDATE, p_any || ', ' || p_idRepresentant, RSP);

    EXCEPTION
        WHEN e_representantNF THEN
            RSP := 'ERROR: ID de representant no trobat';
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_contractesRepresentant', SYSDATE, p_any || ', ' || p_idRepresentant, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_contractesRepresentant', SYSDATE, p_any || ', ' || p_idRepresentant, RSP);

    END lst_contractesRepresentant;

    -- Llista els 10 equips que mes s'han gastat en un any concret
    PROCEDURE lst_top10DespesaEquips(
        p_any IN INTEGER,
        RSP OUT logs.RSP%TYPE)
    IS

        CURSOR c_equips IS
           SELECT nom, valorEconomic
              FROM (SELECT equips.nom,
                  SUM(contractes.valorEconomic) valorEconomic
             FROM equips, contractes
            WHERE equips.idEquip = contractes.equipComprador
              AND EXTRACT(YEAR FROM TO_DATE(contractes.dataInici)) = p_any
              
            GROUP BY equips.nom
            ORDER BY 2 DESC)
            WHERE rownum < 11;

        m_nom equips.nom%TYPE;
        m_valorEconomic contractes.valorEconomic%TYPE;

    BEGIN
        -- Buffer del output
        DBMS_OUTPUT.ENABLE(1000);
        DBMS_OUTPUT.PUT_LINE('=========================');
        DBMS_OUTPUT.PUT_LINE('TOP 10 Despesa per equips');
        DBMS_OUTPUT.PUT_LINE('=========================');

        OPEN c_equips;
        --Per a cada representant, mostrar les dades dels contractes
        LOOP
            FETCH c_equips INTO m_nom, m_valorEconomic;
            EXIT WHEN c_equips%NOTFOUND;

            DBMS_OUTPUT.PUT_LINE(m_nom || ': ' || m_valorEconomic);
        END LOOP;

        CLOSE c_equips;

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('lst_top10DespesaEquips', SYSDATE, p_any, RSP);

    EXCEPTION
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_top10DespesaEquips', SYSDATE, p_any, RSP);

    END lst_top10DespesaEquips;

    -- Llista els 5 jugadors millor valorats per posicio segons ambit, genere i modalitat competicio
    PROCEDURE lst_top5Jugadors(
        p_ambit competicions.ambit%TYPE,
        p_genere competicions.genere%TYPE,
        p_modalitat competicions.modalitat%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS

        CURSOR c_jugadors IS
           SELECT DECODE(posicio,1,'Base',2,'Escorta',3,'Aler',4,'Aler-Pivot','Pivot') posicio, nom, cognoms, valoracio
             FROM (SELECT jugadors.posicio,
                  persones.nom, persones.cognoms,
                  SUM(convocats.valoracio) valoracio
             FROM jugadors, persones, convocats, partits,competicions
            WHERE jugadors.idPersona = persones.idPersona
              AND jugadors.idJugador = convocats.idJugador
              AND partits.idPartit = convocats.idPartit
              AND competicions.idCompeticio = partits.idPartit
              AND competicions.ambit = p_ambit
              AND competicions.genere = p_genere
              AND competicions.modalitat = p_modalitat
              AND jugadors.posicio = 1
            GROUP BY jugadors.posicio,persones.nom,persones.cognoms)
            WHERE rownum < 6
           UNION
           SELECT DECODE(posicio,1,'Base',2,'Escorta',3,'Aler',4,'Aler-Pivot','Pivot') posicio, nom, cognoms, valoracio
             FROM (SELECT jugadors.posicio,
                  persones.nom, persones.cognoms,
                  SUM(convocats.valoracio) valoracio
             FROM jugadors, persones, convocats, partits,competicions
            WHERE jugadors.idPersona = persones.idPersona
              AND jugadors.idJugador = convocats.idJugador
              AND partits.idPartit = convocats.idPartit
              AND competicions.idCompeticio = partits.idPartit
              AND competicions.ambit = p_ambit
              AND competicions.genere = p_genere
              AND competicions.modalitat = p_modalitat
              AND jugadors.posicio = 2
            GROUP BY jugadors.posicio,persones.nom,persones.cognoms)
            WHERE rownum < 6
           UNION
           SELECT DECODE(posicio,1,'Base',2,'Escorta',3,'Aler',4,'Aler-Pivot','Pivot') posicio, nom, cognoms, valoracio
             FROM (SELECT jugadors.posicio,
                  persones.nom, persones.cognoms,
                  SUM(convocats.valoracio) valoracio
             FROM jugadors, persones, convocats, partits,competicions
            WHERE jugadors.idPersona = persones.idPersona
              AND jugadors.idJugador = convocats.idJugador
              AND partits.idPartit = convocats.idPartit
              AND competicions.idCompeticio = partits.idPartit
              AND competicions.ambit = p_ambit
              AND competicions.genere = p_genere
              AND competicions.modalitat = p_modalitat
              AND jugadors.posicio = 3
            GROUP BY jugadors.posicio,persones.nom,persones.cognoms)
            WHERE rownum < 6
           UNION
           SELECT DECODE(posicio,1,'Base',2,'Escorta',3,'Aler',4,'Aler-Pivot','Pivot') posicio, nom, cognoms, valoracio
             FROM (SELECT jugadors.posicio,
                  persones.nom, persones.cognoms,
                  SUM(convocats.valoracio) valoracio
             FROM jugadors, persones, convocats, partits,competicions
            WHERE jugadors.idPersona = persones.idPersona
              AND jugadors.idJugador = convocats.idJugador
              AND partits.idPartit = convocats.idPartit
              AND competicions.idCompeticio = partits.idPartit
              AND competicions.ambit = p_ambit
              AND competicions.genere = p_genere
              AND competicions.modalitat = p_modalitat
              AND jugadors.posicio = 4
            GROUP BY jugadors.posicio,persones.nom,persones.cognoms)
            WHERE rownum < 6
           UNION
           SELECT DECODE(posicio,1,'Base',2,'Escorta',3,'Aler',4,'Aler-Pivot','Pivot') posicio, nom, cognoms, valoracio
             FROM (SELECT jugadors.posicio,
                  persones.nom, persones.cognoms,
                  SUM(convocats.valoracio) valoracio
             FROM jugadors, persones, convocats, partits,competicions
            WHERE jugadors.idPersona = persones.idPersona
              AND jugadors.idJugador = convocats.idJugador
              AND partits.idPartit = convocats.idPartit
              AND competicions.idCompeticio = partits.idPartit
              AND competicions.ambit = p_ambit
              AND competicions.genere = p_genere
              AND competicions.modalitat = p_modalitat
              AND jugadors.posicio = 5
            GROUP BY jugadors.posicio,persones.nom,persones.cognoms)
            WHERE rownum < 6;

        e_ambitCHK EXCEPTION;
        e_genereCHK EXCEPTION;
        e_modalitatCHK EXCEPTION;
        m_posicio VARCHAR2(10);
        m_nom persones.nom%TYPE;
        m_cognoms persones.cognoms%TYPE;
        m_valoracio convocats.valoracio%TYPE;

    BEGIN

        -- Buffer del output
        DBMS_OUTPUT.ENABLE(1000);
        DBMS_OUTPUT.PUT_LINE('==========================');
        DBMS_OUTPUT.PUT_LINE('TOP 5 Jugadors per posicio');
        DBMS_OUTPUT.PUT_LINE('==========================');

        IF (p_ambit NOT IN ('N','C')) THEN
            RAISE e_ambitCHK;
        END IF;

        IF (p_genere NOT IN ('M','F')) THEN
            RAISE e_genereCHK;
        END IF;

        IF (p_modalitat NOT IN ('N','C')) THEN
            RAISE e_modalitatCHK;
        END IF;

        OPEN c_jugadors;
        --Per a cada representant, mostrar les dades dels contractes
        LOOP
            FETCH c_jugadors INTO m_posicio, m_nom, m_cognoms, m_valoracio;
            EXIT WHEN c_jugadors%NOTFOUND;

            DBMS_OUTPUT.PUT_LINE(m_posicio || ', ' || m_nom || ', ' || m_cognoms || ', ' || m_valoracio);
        END LOOP;

        CLOSE c_jugadors;

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('lst_top10DespesaEquips', SYSDATE, p_ambit || ', ' || p_genere || ', ' || p_modalitat, RSP);

    EXCEPTION
        WHEN e_ambitCHK THEN
            RSP := 'ERROR: Ambit [' || p_ambit || '] no vàlid';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_top10DespesaEquips', SYSDATE, p_ambit || ', ' || p_genere || ', ' || p_modalitat, RSP);
        WHEN e_genereCHK THEN
            RSP := 'ERROR: Genere [' || p_genere || '] no vàlid';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_top10DespesaEquips', SYSDATE, p_ambit || ', ' || p_genere || ', ' || p_modalitat, RSP);
        WHEN e_modalitatCHK THEN
            RSP := 'ERROR: Modalitat [' || p_modalitat || '] no vàlida';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_top10DespesaEquips', SYSDATE, p_ambit || ', ' || p_genere || ', ' || p_modalitat, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_top10DespesaEquips', SYSDATE, p_ambit || ', ' || p_genere || ', ' || p_modalitat, RSP);
    END lst_top5Jugadors;

    -- Llistat de jugadors que acaben contracte o no en tenen segons pais, genere i modalitat
    PROCEDURE lst_jugadorsLliures(
        p_idPais IN paisos.idPais%TYPE,
        p_genere competicions.genere%TYPE,
        p_modalitat competicions.modalitat%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS

    CURSOR c_lliures IS
        SELECT persones.nom, persones.cognoms, equips.nom equipActual
          FROM jugadors, persones, contractes, equips
         WHERE jugadors.idPersona = persones.idPersona
           AND jugadors.idJugador = contractes.idContracte
           AND contractes.equipComprador = equips.idEquip
           AND persones.idPais = p_idPais
           AND jugadors.genere = p_genere
           AND jugadors.modalitat = p_modalitat
           AND jugadors.estat = 'A'
           AND EXTRACT(YEAR FROM TO_DATE(contractes.dataFi)) = EXTRACT(YEAR FROM SYSDATE)
         UNION
        SELECT persones.nom, persones.cognoms, '' equipActual
          FROM jugadors, persones
         WHERE jugadors.idPersona = persones.idPersona
           AND jugadors.genere = p_genere
           AND jugadors.modalitat = p_modalitat
           AND jugadors.estat = 'A'
           AND (SELECT COUNT(*) FROM contractes WHERE contractes.idJugador = jugadors.idJugador
           AND SYSDATE BETWEEN contractes.dataInici AND contractes.dataFi) = 0;

    e_paisNF EXCEPTION;
    e_genereCHK EXCEPTION;
    e_modalitatCHK EXCEPTION;
    m_nom persones.nom%TYPE;
    m_cognoms persones.cognoms%TYPE;
    m_equipActual equips.nom%TYPE;
    trobat NUMBER;

    BEGIN

        -- Control per determinar si existeix el codi de pais
        SELECT COUNT(*) INTO trobat FROM paisos WHERE idPais = p_idPais;
        IF (trobat = 0) THEN
            RAISE e_paisNF;
        END IF;

        IF (p_genere NOT IN ('M','F')) THEN
            RAISE e_genereCHK;
        END IF;

        IF (p_modalitat NOT IN ('N','C')) THEN
            RAISE e_modalitatCHK;
        END IF;

        -- Buffer del output
        DBMS_OUTPUT.ENABLE(1000);
        DBMS_OUTPUT.PUT_LINE('=======================================');
        DBMS_OUTPUT.PUT_LINE('Jugadors lliures o que acaben contracte');
        DBMS_OUTPUT.PUT_LINE('=======================================');

        OPEN c_lliures;
        --Per cada jugador processar les dades
        LOOP
            FETCH c_lliures INTO m_nom, m_cognoms, m_equipActual;
            EXIT WHEN c_lliures%NOTFOUND;

            DBMS_OUTPUT.PUT_LINE(m_nom || ', ' || m_cognoms || ', ' || m_equipActual);
        END LOOP;

        CLOSE c_lliures;

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('lst_jugadorsLliures', SYSDATE, p_idPais || ', ' || p_genere || ', ' || p_modalitat, RSP);

    EXCEPTION
     WHEN e_paisNF THEN
            RSP := 'ERROR: Codi de pais [' || p_idPais || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_jugadorsLliures', SYSDATE, p_idPais || ', ' || p_genere || ', ' || p_modalitat, RSP);
        WHEN e_genereCHK THEN
            RSP := 'ERROR: Genere [' || p_genere || '] no vàlid';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_jugadorsLliures', SYSDATE, p_idPais || ', ' || p_genere || ', ' || p_modalitat, RSP);
        WHEN e_modalitatCHK THEN
            RSP := 'ERROR: Modalitat [' || p_modalitat || '] no vàlida';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_jugadorsLliures', SYSDATE, p_idPais || ', ' || p_genere || ', ' || p_modalitat, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_jugadorsLliures', SYSDATE, p_idPais || ', ' || p_genere || ', ' || p_modalitat, RSP);
    END lst_jugadorsLliures;

END pkg_consultes;

