CREATE OR REPLACE PACKAGE pkg_gestioContractes
IS
    RSP logs.RSP%TYPE;

    -- Dona d'alta un contracte en el sistema.
    PROCEDURE ges_altaContracte(
        p_equipComprador IN equips.idEquip%TYPE,
        p_equipVenedor IN equips.idEquip%TYPE,
        p_idJugador IN jugadors.idJugador%TYPE,
        p_dataInici IN contractes.dataInici%TYPE,
        p_dataFi IN contractes.dataFi%TYPE,
        p_salari IN contractes.salari%TYPE,
        p_compensacio IN contractes.compensacio%TYPE,
        p_valorEconomic IN contractes.valorEconomic%TYPE,
        p_moneda IN contractes.moneda%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Modifica les dades d'un contracte.
    PROCEDURE ges_modificaContracte(
        p_idContracte IN contractes.idContracte%TYPE,
        p_dataInici IN contractes.dataInici%TYPE,
        p_dataFi IN contractes.dataFi%TYPE,
        p_salari IN contractes.salari%TYPE,
        p_compensacio IN contractes.compensacio%TYPE,
        p_valorEconomic IN contractes.valorEconomic%TYPE,
        p_moneda IN contractes.moneda%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Esborra un contracte del sistema
    PROCEDURE ges_esborraContracte(
        p_idContracte IN contractes.idContracte%TYPE,
        RSP OUT logs.RSP%TYPE
    );

END pkg_gestioContractes;
/
CREATE OR REPLACE PACKAGE BODY pkg_gestioContractes
IS

    -- Dona d'alta un contracte en el sistema.
    PROCEDURE ges_altaContracte(
        p_equipComprador IN equips.idEquip%TYPE,
        p_equipVenedor IN equips.idEquip%TYPE,
        p_idJugador IN jugadors.idJugador%TYPE,
        p_dataInici IN contractes.dataInici%TYPE,
        p_dataFi IN contractes.dataFi%TYPE,
        p_salari IN contractes.salari%TYPE,
        p_compensacio IN contractes.compensacio%TYPE,
        p_valorEconomic IN contractes.valorEconomic%TYPE,
        p_moneda IN contractes.moneda%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_equipVNF EXCEPTION;
        e_equipCNF EXCEPTION;
        e_jugadorNF EXCEPTION;
        e_null EXCEPTION;
        e_datesCHK EXCEPTION;
        e_mateixEquip EXCEPTION;
        trobat NUMBER;
        m_genere jugadors.genere%TYPE;
        m_modalitat jugadors.modalitat%TYPE;
        m_anyInicial NUMBER;
        m_anyFinal NUMBER;
        m_idPais paisos.idPais%TYPE;
        m_avgSalari contractes.salari%TYPE;
        m_nomPais paisos.nom%TYPE;
    BEGIN
        -- Control equip venedor existent
        IF (p_equipVenedor IS NOT NULL) THEN
            SELECT COUNT(*) INTO trobat FROM equips WHERE idEquip = p_equipVenedor;
            IF (trobat = 0) THEN
                RAISE e_equipVNF;
            END IF;
        END IF;

        -- Control equip comprador existent
        SELECT COUNT(*) INTO trobat FROM equips WHERE idEquip = p_equipComprador;
        IF (trobat = 0) THEN
            RAISE e_equipCNF;
        END IF;

        -- Control jugador existent
        SELECT COUNT(*) INTO trobat FROM jugadors WHERE idJugador = p_idJugador;
        IF (trobat = 0) THEN
            RAISE e_jugadorNF;
        END IF;

        -- control dades obligatories
        IF (p_dataInici IS NULL OR p_dataFi IS NULL OR p_salari IS NULL OR p_moneda IS NULL) THEN
            RAISE e_null;
        END IF;

        -- check data inici menor que data fi
        IF (p_dataInici > p_dataFi) THEN
            RAISE e_datesCHK;
        END IF;

        -- Controlar que el equip comprador i venedor no siguin el mateix
        IF (NVL(p_equipVenedor,'') = NVL(p_equipComprador,'')) THEN
            RAISE e_mateixEquip;
        END IF;

        -- Si no hi ha cap error, podem crear el contracte
        INSERT INTO contractes (equipComprador, equipVenedor, idJugador,
                                dataInici, dataFi, salari, compensacio, valorEconomic, moneda)
        VALUES (p_equipComprador, p_equipVenedor, p_idJugador,
                p_dataInici, p_dataFi, p_salari, NVL(p_compensacio,0), NVL(p_valorEconomic,0), p_moneda);

        -- Actualitza les dades de dwh_millorPagats.
        -- El jugador ms ben pagat de cada modalitat i gnere.
        -- Primer actualitzar per si ja existeix registre per jugador.
        -- No cal condicionar la cerca per modalitat o genere ja que un en cas de canviar es considera un altre jugador.
        UPDATE dwh_millorPagats
           SET salari = p_salari
         WHERE idJugador = p_idJugador;

        -- Si no existeix aquesta estadistica al DWH la donem d'alta.
        IF (SQL%ROWCOUNT = 0) THEN
            INSERT INTO dwh_millorPagats (idJugador, nom, cognoms, modalitat, genere, salari)
            SELECT jugadors.idJugador, persones.nom, persones.cognoms, jugadors.modalitat, jugadors.genere, p_salari
              FROM persones, jugadors
             WHERE persones.idPersona = jugadors.idPersona
               AND jugadors.idJugador = p_idJugador;
        END IF;

        -- Actualitza les dades de dwh_totalGuanys.
        -- El jugador amb ms guanys acumulats al llarg de tota la seva carrera esportiva.
        -- Considerarem que els contractes duren anys de forma entera.
        UPDATE dwh_totalGuanys
           SET guanys = p_salari * FLOOR(MONTHS_BETWEEN(p_dataFi, p_dataInici) / 12)
         WHERE dwh_totalGuanys.idJugador = p_idJugador;

        -- Si no existeix aquesta estadistica al DWH la donem d'alta.
        IF (SQL%ROWCOUNT = 0) THEN
            INSERT INTO dwh_totalGuanys (idJugador, nom, cognoms, guanys)
            SELECT jugadors.idJugador, persones.nom, persones.cognoms, p_salari * FLOOR(MONTHS_BETWEEN(p_dataFi, p_dataInici) / 12)
              FROM persones, jugadors
             WHERE persones.idPersona = jugadors.idPersona
               AND jugadors.idJugador = p_idJugador;
        END IF;

        -- Actualitza les dades de dwh_souMigJugadors
        -- Donat un pas, una temporada, un gnere i una modalitat, el sou mig anual dels jugadors.

        -- Recuperem dades del jugador
        SELECT jugadors.genere, jugadors.modalitat, persones.idPais, paisos.nom
          INTO m_genere, m_modalitat, m_idPais, m_nomPais
          FROM jugadors, persones, paisos
         WHERE jugadors.idPersona = persones.idPersona
           AND jugadors.idJugador = p_idJugador
           AND persones.idPais = paisos.idPais;

        -- Actualitzar les estadistiques per cada any que impliqui el contracte
        m_anyInicial := EXTRACT(YEAR FROM p_dataInici);
        m_anyFinal := EXTRACT(YEAR FROM p_dataFi);

        WHILE m_anyInicial < m_anyFinal
        LOOP
            -- Calcular el sou mig per aquest any, pais, genere i modalitat, segons els contractes que tinguin els jugadors.
            -- Asumirem que un jugador nomes pot tindre un contracte en un determinat moment
            SELECT AVG(salari)
              INTO m_avgSalari
              FROM contractes, jugadors, persones
             WHERE contractes.idJugador = jugadors.idJugador
               AND jugadors.idPersona = persones.idPersona
               AND persones.idPais = m_idPais
               AND jugadors.genere = m_genere
               AND jugadors.modalitat = m_modalitat
               AND m_anyInicial BETWEEN EXTRACT(YEAR FROM contractes.dataInici) AND EXTRACT(YEAR FROM contractes.dataFi);

            -- Actualitzem les estadistiques
            UPDATE dwh_souMigJugadors
               SET souMig = NVL(m_avgSalari,0)
             WHERE idPais = m_idPais
               AND anyTemporada = m_anyInicial
               AND genere = m_genere
               AND modalitat = m_modalitat;

            -- Si no existeix aquesta estadistica al DWH la donem d'alta.
            IF (SQL%ROWCOUNT = 0) THEN
                INSERT INTO dwh_souMigJugadors (idPais, nomPais, anyTemporada, genere, modalitat, souMig)
                VALUES (m_idPais, m_nomPais, m_anyInicial, m_genere, m_modalitat, NVL(m_avgSalari,0));
            END IF;

            m_anyInicial := m_anyInicial + 1;
        END LOOP;

        DBMS_OUTPUT.PUT_LINE('Contracte creat');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_altaContracte', SYSDATE,
                p_equipComprador || ', ' || p_equipVenedor || ', ' || p_idJugador || ', ' || p_dataInici || ', ' ||
                p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);

    EXCEPTION
        WHEN e_equipVNF THEN
            RSP := 'ERROR: Equip venedor no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaContracte', SYSDATE,
                    p_equipComprador || ', ' || p_equipVenedor || ', ' || p_idJugador || ', ' || p_dataInici || ', ' ||
                    p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);
        WHEN e_equipCNF THEN
            RSP := 'ERROR: Equip comprador no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaContracte', SYSDATE,
                    p_equipComprador || ', ' || p_equipVenedor || ', ' || p_idJugador || ', ' || p_dataInici || ', ' ||
                    p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);
        WHEN e_jugadorNF THEN
            RSP := 'ERROR: Jugador [' || p_idJugador || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaContracte', SYSDATE,
                    p_equipComprador || ', ' || p_equipVenedor || ', ' || p_idJugador || ', ' || p_dataInici || ', ' ||
                    p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);
        WHEN e_null THEN
            RSP := 'ERROR: Data inici, data fi, salari i moneda son obligatoris';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaContracte', SYSDATE,
                    p_equipComprador || ', ' || p_equipVenedor || ', ' || p_idJugador || ', ' || p_dataInici || ', ' ||
                    p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);
        WHEN e_datesCHK THEN
            RSP := 'ERROR: La data fi ha de ser posterior o igual a la data inici';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaContracte', SYSDATE,
                    p_equipComprador || ', ' || p_equipVenedor || ', ' || p_idJugador || ', ' || p_dataInici || ', ' ||
                    p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);
        WHEN e_mateixEquip THEN
            RSP := 'ERROR: Equip comprador i venedor han de ser diferents';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
           VALUES ('ges_altaContracte', SYSDATE,
                    p_equipComprador || ', ' || p_equipVenedor || ', ' || p_idJugador || ', ' || p_dataInici || ', ' ||
                    p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en alta de contracte ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaContracte', SYSDATE,
                    p_equipComprador || ', ' || p_equipVenedor || ', ' || p_idJugador || ', ' || p_dataInici || ', ' ||
                    p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);

    END ges_altaContracte;

    -- Modifica les dades d'un contracte.
    PROCEDURE ges_modificaContracte(
        p_idContracte IN contractes.idContracte%TYPE,
        p_dataInici IN contractes.dataInici%TYPE,
        p_dataFi IN contractes.dataFi%TYPE,
        p_salari IN contractes.salari%TYPE,
        p_compensacio IN contractes.compensacio%TYPE,
        p_valorEconomic IN contractes.valorEconomic%TYPE,
        p_moneda IN contractes.moneda%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_contracteNF EXCEPTION;
        e_null EXCEPTION;
        e_datesCHK EXCEPTION;
        trobat NUMBER;
        m_genere jugadors.genere%TYPE;
        m_modalitat jugadors.modalitat%TYPE;
        m_anyInicial NUMBER;
        m_anyFinal NUMBER;
        m_idPais paisos.idPais%TYPE;
        m_avgSalari contractes.salari%TYPE;
        m_nomPais paisos.nom%TYPE;
        m_idJugador jugadors.idJugador%TYPE;
        m_dataInici contractes.dataInici%TYPE;
        m_dataFi contractes.dataFi%TYPE;
    BEGIN
        -- Control contracte existent
        SELECT COUNT(*) INTO trobat FROM contractes WHERE idContracte = p_idContracte;
        IF (trobat = 0) THEN
            RAISE e_contracteNF;
        END IF;

        -- control dades obligatories
        IF (p_dataInici IS NULL OR p_dataFi IS NULL OR p_salari IS NULL OR p_moneda IS NULL) THEN
            RAISE e_null;
        END IF;

        -- check data inici menor que data fi
        IF (p_dataInici > p_dataFi) THEN
            RAISE e_datesCHK;
        END IF;

        -- Si no hi ha cap error, podem actualitzar les dades del contracte
        UPDATE contractes
           SET dataInici = p_dataInici,
               dataFi = p_dataFi,
               salari = p_salari,
               compensacio = NVL(p_compensacio,0),
               valorEconomic = NVL(p_valorEconomic,0),
               moneda = p_moneda
         WHERE idContracte = p_idContracte;

        -- Dades del contracte, necessaries per poder actualitzar les estadistiques del DWH
        SELECT idJugador, dataInici, dataFi
          INTO m_idJugador, m_dataInici, m_dataFi
          FROM contractes
         WHERE idContracte = p_idContracte;
 
        -- Actualitza les dades de dwh_millorPagats.
        -- El jugador ms ben pagat de cada modalitat i gnere.
        -- Primer actualitzar per si ja existeix registre per jugador.
        -- No cal condicionar la cerca per modalitat o genere ja que un en cas de canviar es considera un altre jugador.
        UPDATE dwh_millorPagats
           SET salari = p_salari
         WHERE idJugador = m_idJugador;

        -- Actualitza les dades de dwh_totalGuanys.
        -- El jugador amb ms guanys acumulats al llarg de tota la seva carrera esportiva.
        -- Considerarem que els contractes duren anys de forma entera.
        UPDATE dwh_totalGuanys
           SET guanys = p_salari * FLOOR(MONTHS_BETWEEN(p_dataFi, p_dataInici) / 12)
         WHERE dwh_totalGuanys.idJugador = m_idJugador;

        -- Actualitza les dades de dwh_souMigJugadors
        -- Donat un pas, una temporada, un gnere i una modalitat, el sou mig anual dels jugadors.

        -- Recuperem dades del jugador
        SELECT jugadors.genere, jugadors.modalitat, persones.idPais, paisos.nom
          INTO m_genere, m_modalitat, m_idPais, m_nomPais
          FROM jugadors, persones, paisos
         WHERE jugadors.idPersona = persones.idPersona
           AND jugadors.idJugador = m_idJugador
           AND persones.idPais = paisos.idPais;

        -- Actualitzar les estadistiques per cada any que impliqui el contracte
        m_anyInicial := EXTRACT(YEAR FROM p_dataInici);
        m_anyFinal := EXTRACT(YEAR FROM p_dataFi);

        WHILE m_anyInicial < m_anyFinal
        LOOP
            -- Calcular el sou mig per aquest any, pais, genere i modalitat, segons els contractes que tinguin els jugadors.
            -- Asumirem que un jugador nomes pot tindre un contracte en un determinat moment
            SELECT AVG(salari)
              INTO m_avgSalari
              FROM contractes, jugadors, persones
             WHERE contractes.idJugador = jugadors.idJugador
               AND jugadors.idPersona = persones.idPersona
               AND persones.idPais = m_idPais
               AND jugadors.genere = m_genere
               AND jugadors.modalitat = m_modalitat
               AND m_anyInicial BETWEEN EXTRACT(YEAR FROM contractes.dataInici) AND EXTRACT(YEAR FROM contractes.dataFi);

            -- Actualitzem les estadistiques
            UPDATE dwh_souMigJugadors
               SET souMig = NVL(m_avgSalari,0)
             WHERE idPais = m_idPais
               AND anyTemporada = m_anyInicial
               AND genere = m_genere
               AND modalitat = m_modalitat;

            -- Si no existeix aquesta estadistica al DWH la donem d'alta.
            IF (SQL%ROWCOUNT = 0) THEN
                INSERT INTO dwh_souMigJugadors (idPais, nomPais, anyTemporada, genere, modalitat, souMig)
                VALUES (m_idPais, m_nomPais, m_anyInicial, m_genere, m_modalitat, NVL(m_avgSalari,0));
            END IF;

            m_anyInicial := m_anyInicial + 1;
        END LOOP;

        DBMS_OUTPUT.PUT_LINE('Contracte modificat');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_modificaContracte', SYSDATE,
                p_idContracte || ', ' || p_dataInici || ', ' ||
                p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);

    EXCEPTION
        WHEN e_contracteNF THEN
            RSP := 'ERROR: ID de contracte [' || p_idContracte || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaContracte', SYSDATE,
                    p_idContracte || ', ' || p_dataInici || ', ' ||
                    p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);
        WHEN e_null THEN
            RSP := 'ERROR: Data inici, data fi, salari i moneda son obligatoris';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaContracte', SYSDATE,
                    p_idContracte || ', ' || p_dataInici || ', ' ||
                    p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);
        WHEN e_datesCHK THEN
            RSP := 'ERROR: La data fi ha de ser posterior o igual a la data inici';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaContracte', SYSDATE,
                    p_idContracte || ', ' || p_dataInici || ', ' ||
                    p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en modificacio de contracte ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaContracte', SYSDATE,
                    p_idContracte || ', ' || p_dataInici || ', ' ||
                    p_dataFi || ', ' || p_salari || ', ' || p_compensacio || ', ' || p_valorEconomic || ', ' || p_moneda, RSP);

    END ges_modificaContracte;

    -- Esborra un contracte del sistema
    PROCEDURE ges_esborraContracte(
        p_idContracte IN contractes.idContracte%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_contracteNF EXCEPTION;
        trobat NUMBER;
        m_idJugador jugadors.idJugador%TYPE;
        m_genere jugadors.genere%TYPE;
        m_modalitat jugadors.modalitat%TYPE;
        m_anyInicial NUMBER;
        m_anyFinal NUMBER;
        m_idPais paisos.idPais%TYPE;
        m_avgSalari contractes.salari%TYPE;
        m_nomPais paisos.nom%TYPE;
        m_dataInici contractes.dataInici%TYPE;
        m_dataFi contractes.dataFi%TYPE;

    BEGIN
        -- Control contracte existent
        SELECT COUNT(*) INTO trobat FROM contractes WHERE idContracte = p_idContracte;
        IF (trobat = 0) THEN
            RAISE e_contracteNF;
        END IF;

        -- Dades del contracte, necessaries per poder actualitzar les estadistiques del DWH
        SELECT idJugador, dataInici, dataFi
          INTO m_idJugador, m_dataInici, m_dataFi
          FROM contractes
         WHERE idContracte = p_idContracte;

        -- Esborrar el contracte
        DELETE FROM contractes WHERE idContracte = p_idContracte;

        -- Actualitza les dades de dwh_millorPagats
        -- El jugador ms ben pagat de cada modalitat i gnere.
        UPDATE dwh_millorPagats
           SET salari = NVL((SELECT MAX(salari) FROM contractes WHERE idJugador = m_idJugador),0)
         WHERE idJugador = m_idJugador;

        -- Actualitza les dades de dwh_totalGuanys.
        -- El jugador amb ms guanys acumulats al llarg de tota la seva carrera esportiva.
        -- Considerarem que els contractes duren anys de forma entera.
        UPDATE dwh_totalGuanys
           SET guanys = NVL((SELECT SUM(salari * FLOOR(MONTHS_BETWEEN(dataFi, dataInici) / 12)) FROM contractes WHERE idJugador = m_idJugador),0)
         WHERE dwh_totalGuanys.idJugador = m_idJugador;

        -- Actualitza les dades de dwh_souMigJugadors
        -- Donat un pas, una temporada, un gnere i una modalitat, el sou mig anual dels jugadors.
        -- Recuperem dades del jugador
        SELECT jugadors.genere, jugadors.modalitat, persones.idPais, paisos.nom
          INTO m_genere, m_modalitat, m_idPais, m_nomPais
          FROM jugadors, persones, paisos
         WHERE jugadors.idPersona = persones.idPersona
           AND jugadors.idJugador = m_idJugador
           AND persones.idPais = paisos.idPais;

        -- Actualitzar les estadistiques per cada any que impliqui el contracte
        m_anyInicial := EXTRACT(YEAR FROM m_dataInici);
        m_anyFinal := EXTRACT(YEAR FROM m_dataFi);

        WHILE m_anyInicial < m_anyFinal
        LOOP
            -- Calcular el sou mig per aquest any, pais, genere i modalitat, segons els contractes que tinguin els jugadors.
            -- Asumirem que un jugador nomes pot tindre un contracte en un determinat moment
            SELECT AVG(salari)
              INTO m_avgSalari
              FROM contractes, jugadors, persones
             WHERE contractes.idJugador = jugadors.idJugador
               AND jugadors.idPersona = persones.idPersona
               AND persones.idPais = m_idPais
               AND jugadors.genere = m_genere
               AND jugadors.modalitat = m_modalitat
               AND m_anyInicial BETWEEN EXTRACT(YEAR FROM contractes.dataInici) AND EXTRACT(YEAR FROM contractes.dataFi);

            -- Actualitzem les estadistiques
            UPDATE dwh_souMigJugadors
               SET souMig = NVL(m_avgSalari,0)
             WHERE idPais = m_idPais
               AND anyTemporada = m_anyInicial
               AND genere = m_genere
               AND modalitat = m_modalitat;

            -- Si no existeix aquesta estadistica al DWH la donem d'alta.
            IF (SQL%ROWCOUNT = 0) THEN
                INSERT INTO dwh_souMigJugadors (idPais, nomPais, anyTemporada, genere, modalitat, souMig)
                VALUES (m_idPais, m_nomPais, m_anyInicial, m_genere, m_modalitat, NVL(m_avgSalari,0));
            END IF;

            m_anyInicial := m_anyInicial + 1;
        END LOOP;

        DBMS_OUTPUT.PUT_LINE('Contracte esborrat');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_esborraContracte', SYSDATE, p_idContracte, RSP);

    EXCEPTION
        WHEN e_contracteNF THEN
            RSP := 'ERROR: ID de contracte [' || p_idContracte || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_esborraContracte', SYSDATE, p_idContracte, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en esborrat de contracte ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_esborraContracte', SYSDATE, p_idContracte, RSP);

    END ges_esborraContracte;

END pkg_gestioContractes;